@@ -1,4 +1,4 @@
 from famegui import app
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     app.run()
