@@ -5,11 +5,11 @@ from PySide2 import QtCore
 from famegui import models
 from famegui.agent_controller import AgentController
 from famegui.appworkingdir import AppWorkingDir
-from famegui.agent_validator import AgentValidator
+import fameio.source.validator as fameio
 
 
 class ProjectProperties(QtCore.QObject):
-    """ Class used to attach extra properties to a scenario model and signal when they change """
+    """Class used to attach extra properties to a scenario model and signal when they change"""
 
     changed = QtCore.Signal()
 
@@ -17,23 +17,17 @@ class ProjectProperties(QtCore.QObject):
         super().__init__()
         self.reset()
 
-    def reset(self, schema: models.Schema = None, file_path=""):
+    def reset(self, file_path=""):
         self._has_unsaved_changes = False
         self._file_path = file_path
-        self._schema = schema
         self._validation_errors = []
         self.changed.emit()
 
-    @property
-    def is_open(self) -> bool:
-        return self._schema is not None
-
     @property
     def has_unsaved_changes(self) -> bool:
         return self._has_unsaved_changes
 
     def set_unsaved_changes(self, has_changes: bool = True):
-        assert self.is_open
         if self._has_unsaved_changes != has_changes:
             self._has_unsaved_changes = has_changes
             self.changed.emit()
@@ -42,37 +36,32 @@ class ProjectProperties(QtCore.QObject):
     def file_path(self) -> str:
         return self._file_path
 
-    def set_file_path(self, file_path: str):
-        assert self.is_open
+    def set_file_path(self, file_path: str) -> None:
         if self._file_path != file_path:
             self._file_path = file_path
             self.changed.emit()
 
-    @property
-    def schema(self) -> models.Schema:
-        assert self._schema is not None
-        return self._schema
-
-    @property
-    def schema_path(self) -> str:
-        return self.schema.file_path
-
     @property
     def is_validation_successful(self) -> bool:
-        #assert self.is_open
         return len(self._validation_errors) == 0
 
     @property
     def validation_errors(self) -> typing.List[str]:
-        assert self.is_open
         return self._validation_errors
 
-    def add_validation_error(self, msg: str):
+    def clear_validation_errors(self) -> None:
+        was_invalid = not self.is_validation_successful
+        self._validation_errors = []
+        if was_invalid:
+            logging.info("schema validation status changed to valid")
+            self.changed.emit()
+
+    def set_validation_error(self, msg: str) -> None:
+        assert msg != ""
         was_valid = self.is_validation_successful
-        self._validation_errors.append(msg)
+        self._validation_errors = [msg]
         if was_valid:
-            logging.info(
-                "schema validation status changed from successful to failed")
+            logging.info("schema validation status changed to invalid")
             self.changed.emit()
 
 
@@ -83,7 +72,8 @@ class MainController(QtCore.QObject):
     agent_added = QtCore.Signal(AgentController)
     # new contract added
     contract_added = QtCore.Signal(
-        AgentController, AgentController, models.Contract)  # sender, receiver, contract
+        AgentController, AgentController, models.Contract
+    )  # sender, receiver, contract
 
     def __init__(self, working_dir: AppWorkingDir):
         super().__init__()
@@ -94,15 +84,15 @@ class MainController(QtCore.QObject):
         self._contracts = {}
         self._project_properties = ProjectProperties()
 
-    def reset(self):
+    def reset(self, scenario: models.Scenario = None) -> None:
         self._agents = {}
         self._contracts = {}
+        self._scenario_model = scenario
         self._project_properties.reset()
-        self._scenario_model = None
 
     @property
     def is_open(self) -> bool:
-        return self._scenario_model is not None and self.project_properties.is_open
+        return self._scenario_model is not None
 
     @property
     def has_unsaved_changes(self) -> bool:
@@ -114,20 +104,24 @@ class MainController(QtCore.QObject):
 
     @property
     def schema(self) -> models.Schema:
-        return self._project_properties.schema
+        return self.scenario.schema
 
     @property
     def scenario(self) -> models.Scenario:
         return self._scenario_model
 
-    def set_scenario_properties(self, props: models.Scenario):
-        has_changed = self._scenario_model.set_properties(props)
+    def update_scenario_properties(self, props: models.GeneralProperties):
+        has_changed = self._scenario_model.update_properties(props)
         if has_changed:
             self._project_properties.set_unsaved_changes(True)
 
     @property
     def can_export_protobuf(self) -> bool:
-        return self.is_open and self.project_properties.is_validation_successful and self.has_unsaved_changes == False
+        return (
+            self.is_open
+            and self.project_properties.is_validation_successful
+            and not self.has_unsaved_changes
+        )
 
     @property
     def agent_count(self) -> int:
@@ -152,7 +146,7 @@ class MainController(QtCore.QObject):
             item_size = a.scene_item.boundingRect().width()
             left = a.scene_item.x() - margin
             right = a.scene_item.x() + item_size + margin
-            top = a.scene_item.y() + - margin
+            top = a.scene_item.y() + -margin
             bottom = a.scene_item.y() + item_size + margin
             if left < rect.left():
                 rect.setLeft(left)
@@ -171,7 +165,7 @@ class MainController(QtCore.QObject):
         # so we check for possible conflict and solve it
         if new_id in self._agents:
             for i in range(1, len(self._agents) + 2):
-                if not i in self._agents:
+                if i not in self._agents:
                     new_id = i
                     break
         logging.debug("generated new agent id {}".format(new_id))
@@ -189,51 +183,59 @@ class MainController(QtCore.QObject):
     def add_new_agent(self, agent_model: models.Agent, x: int, y: int):
         assert self.is_open
         agent_model.set_display_xy(x, y)
-        self._validate_and_create_agent_controller(agent_model)
+        self._create_agent_controller(agent_model)
         self._scenario_model.add_agent(agent_model)
         self._project_properties.set_unsaved_changes(True)
-        logging.info("created new agent {} of type '{}'".format(
-            agent_model.id_str, agent_model.type_name))
+        self.revalidate_scenario()
+        logging.info(
+            "created new agent {} of type '{}'".format(
+                agent_model.display_id, agent_model.type_name
+            )
+        )
 
-    def _validate_and_create_agent_controller(self, agent_model: models.Agent):
+    def _create_agent_controller(self, agent_model: models.Agent):
         assert self.is_open
 
         # accept to add the agent even if invalid
         agent_ctrl = AgentController(agent_model)
         self._agents[agent_ctrl.id] = agent_ctrl
         agent_ctrl.model_was_modified.connect(
-            self._project_properties.set_unsaved_changes)
+            self._project_properties.set_unsaved_changes
+        )
 
-        # perform schema validation
-        validator = AgentValidator(self.schema, self._working_dir)
-        ok, err_msg = validator.validate_agent(agent_ctrl.model)
-        if not ok:
-            logging.warning(err_msg)
-            self._project_properties.add_validation_error(err_msg)
-
-        logging.info("new agent {} added".format(agent_ctrl.id_str))
+        logging.info("new agent {} added".format(agent_ctrl.display_id))
         self.agent_added.emit(agent_ctrl)
 
     def add_new_contract(self, contract_model: models.Contract):
         self._scenario_model.add_contract(contract_model)
         self._create_contract_model(contract_model)
         self._project_properties.set_unsaved_changes(True)
-        logging.info("created new contract '{}' between {} and {}".format(
-            contract_model.product_name, contract_model.sender_id_str, contract_model.receiver_id_str))
+        self.revalidate_scenario()
+        logging.info(
+            "created new contract '{}' between {} and {}".format(
+                contract_model.product_name,
+                contract_model.display_sender_id,
+                contract_model.display_receiver_id,
+            )
+        )
 
     def _create_contract_model(self, contract: models.Contract):
         assert self.is_open
-        # should be enforced when creating / reading the contract
-        assert contract.sender_id != contract.receiver_id
 
         # validate sender / receiver are known
         if contract.sender_id not in self._agents:
-            raise ValueError("can't add contract '{}' because sender agent id '{}' is unknown".format(
-                contract.product_name, contract.sender_id))
+            raise ValueError(
+                "can't add contract '{}' because sender agent id '{}' is unknown".format(
+                    contract.product_name, contract.sender_id
+                )
+            )
 
         if contract.receiver_id not in self._agents:
-            raise ValueError("can't add contract '{}' because receiver agent id '{}' is unknown".format(
-                contract.product_name, contract.receiver_id))
+            raise ValueError(
+                "can't add contract '{}' because receiver agent id '{}' is unknown".format(
+                    contract.product_name, contract.receiver_id
+                )
+            )
 
         sender_ctrl = self._agents[contract.sender_id]
         receiver_ctrl = self._agents[contract.receiver_id]
@@ -244,40 +246,34 @@ class MainController(QtCore.QObject):
 
         self.contract_added.emit(sender_ctrl, receiver_ctrl, contract)
 
-    def create_new_scenario(self, schema: models.Schema):
-        # don't call this function when a scenario already exists to avoid accidental loss of data
-        assert not self.is_open
+    def revalidate_scenario(self):
+        assert self._scenario_model is not None
+        try:
+            fameio.SchemaValidator.ensure_is_valid_scenario(self._scenario_model)
+            self._project_properties.clear_validation_errors()
+        except fameio.ValidationException as e:
+            err_msg = str(e)
+            logging.warning("failed to validate the scenario: {}".format(err_msg))
+            self._project_properties.set_validation_error(err_msg)
 
-        self._scenario_model = models.Scenario()
-        self._scenario_model.set_schema_path(schema.file_path)
-        self._scenario_model.properties.init_missing_values()
-        self._project_properties.reset(schema)
-        self._project_properties.set_unsaved_changes(True)
-
-    def open_scenario(self, scenario: models.Scenario, file_path: str, schema: models.Schema):
-        logging.debug("opening new scenario and schema")
+    def init_scenario_model(self, scenario: models.Scenario, file_path: str):
+        logging.debug("opening new scenario")
         self.reset()
-        scenario.set_schema_path(schema.file_path)
 
         try:
             self._scenario_model = scenario
-            changed_done = False
-            if self._scenario_model.properties.init_missing_values():
-                logging.warning(
-                    "the loaded scenario file was updated to contain all the required properties")
-                changed_done = True
-
-            self._project_properties.reset(schema, file_path)
-            self._project_properties.set_unsaved_changes(changed_done)
+            self._project_properties.reset(file_path)
+            self._project_properties.set_unsaved_changes(True)
 
             # process and validate the scenario
-            for a in self._scenario_model.agents.values():
-                self._validate_and_create_agent_controller(a)
+            for a in self._scenario_model.agents:
+                self._create_agent_controller(a)
             for c in self._scenario_model.contracts:
                 self._create_contract_model(c)
-        except:
-            logging.warning(
-                "failed to open the scenario with the given schema")
+
+            self.revalidate_scenario()
+        except Exception as e:
+            logging.error("failed to init the given scenario: {}".format(e))
             self.reset()
             raise
 
@@ -286,9 +282,8 @@ class MainController(QtCore.QObject):
 
     def save_to_file(self, file_path):
         assert self.is_open
-        logging.info("saving scenario to file {}".format(file_path))
-        models.ScenarioLoader.save_to_yaml_file(
-            self._scenario_model, file_path)
+        models.ScenarioLoader.save_to_yaml_file(self._scenario_model, file_path)
         # update status
         self._project_properties.set_unsaved_changes(False)
         self._project_properties.set_file_path(file_path)
+        self.revalidate_scenario()
