@@ -6,7 +6,7 @@ from PySide2 import QtCore
 
 
 class UserSettings:
-    """ Class to load and save user settings on the local host """
+    """Class to load and save user settings on the local host"""
 
     def __init__(self):
         self._settings = QtCore.QSettings("FAME", "famegui")
@@ -19,8 +19,7 @@ class UserSettings:
         if dir_path is None:
             logging.warning("path to working dir is not defined")
         elif not os.path.isdir(dir_path):
-            logging.warning(
-                "deleting invalid path to working dir: {}".format(dir_path))
+            logging.warning("deleting invalid path to working dir: {}".format(dir_path))
             self._settings.remove(self._KEY_WORKING_DIR)
             dir_path = None
         return dir_path
@@ -30,5 +29,6 @@ class UserSettings:
         logging.info("configuring new working dir: {}".format(path))
         if not os.path.isdir(path):
             raise ValueError(
-                "Can't use '{}' as new working directory: invalid path.".format(path))
+                "Can't use '{}' as new working directory: invalid path.".format(path)
+            )
         return self._settings.setValue(self._KEY_WORKING_DIR, path)
